;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;                                                                             ;
;                                                                             ; 
;       This lisp first appeared in the December 1991 issue of Cadence        ;
;       Magazine.  The original "OSM.LSP" had 8 snap functions. I added       ;
;       two more snap functions to this lisp file.  It will draw a snap       ;
;       "menu" on your screen using grdraw. You will see 10 "boxes" appear    ;
;       at the center of your crosshairs.  These are your snap functions.     ;
;       From top left you will have snap tangent, endpoint, intersection,     ;
;       nearest, midpoint and quadrant.  The bottom row from left to right,   ;
;       is snap perpendicular, node, center and insert. Just move your        ;
;       crosshairs to that snap function, pick it with your mouse, and        ;
;       you will be in that snap mode. After you pick it, the menu will       ;
;       disappear.                                                            ;               
;                                                                             ;
;                            Ronald A. Kalister                               ;
;                             Brunswick, Ohio.                                ; 
;                                                                             ;
;                                                                             ;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


(princ "Loading osm10...")


(defun DEFAULTS ()                    ;sets pop-up menu's color and size
 (setq col 7)                         ;set menu color white
 (setq menu_size 4)                   ;set menu size approx.
                                      ;one-fourth of the drawing area
 )

(defun DOSM (scp sc col)              ;function draw Osnap menu
 (setq lx (- (car scp) (/ sc 1.0)))   ;get max. x,y points of menu box.
 (setq hx (+ (car scp) (/ sc 1.0)))   
 (setq ly (- (cadr scp) (/ sc 2.0)))   
 (setq hy (+ (cadr scp) (/ sc 2.0)))  
 (setq x1 (+ lx (/ sc 2.0)))          ;get coordinates of small menu boxes
 (setq x2 (+ x1 (/ sc 2.0)))
 (setq x3 (+ x2 (/ sc 2.0)))
 (setq x4 (+ hx (caddr scp) (/ sc 2.0)))
 (setq x5 (- lx (caddr scp) (/ sc 2.0)))
 (setq y1 (+ ly (/ sc 2.0)))         
 (grdraw (list lx ly) (list hx ly) col) 
 (grdraw (list hx ly) (list hx hy) col) 
 (grdraw (list hx hy) (list lx hy) col) 
 (grdraw (list lx hy) (list lx ly) col) 
 (grdraw (list x1 ly) (list x1 hy) col) 
 (grdraw (list x2 ly) (list x2 hy) col) 
 (grdraw (list x3 ly) (list x3 hy) col) 
 (grdraw (list x4 y1) (list x4 hy) col) 
 (grdraw (list x5 y1) (list x5 hy) col) 
 (grdraw (list x5 hy) (list x4 hy) col) 
 (grdraw (list x5 y1) (list x4 y1) col) 
 (grdraw (list lx y1) (list hx y1) col) 
 (setq ofst (/ sc 20.0))
 (grdraw (list (+ lx ofst) (- ly ofst)) (list (+ hx ofst) (- ly ofst)) col)
 (grdraw (list (+ hx ofst) (- ly ofst)) (list (+ hx ofst) (- y1 ofst)) col)
 (grdraw (list (+ hx ofst) (- y1 ofst)) (list (+ x4 ofst) (- y1 ofst)) col)
 (grdraw (list (+ x4 ofst) (- y1 ofst)) (list (+ x4 ofst) (- hy ofst)) col) 
 (setq d4 (abs (/ (- hx lx) 16.0)))
 (pp lx ly x1 y1)                    ;draw per menu  
 (nd x1 ly x2 y1)                    ;draw nod menu  
 (cn x2 ly x3 y1)                    ;draw cen menu   
 (nd x2 ly x3 y1)                    ;draw a nod inside cen menu
 (in x3 ly hx y1)                    ;draw ins menu
 (en lx y1 x1 hy)                    ;draw end menu
 (it x1 y1 x2 hy)                    ;draw int menu
 (ne x2 y1 x3 hy)                    ;draw near menu
 (md x3 y1 hx hy)                    ;draw mid menu
 (tn x5 y1 lx hy)                    ;draw tan menu
 (qd hx y1 x4 hy)                    ;draw quad menu
)

(defun C:OSM10 (/ ppp px py)           ;function Osnap menu-main function
 (defaults)                          ;set defaults
 (setq sc (/ (getvar "VIEWSIZE") menu_size)) ;set menu box size
 (setq scp (cadr (grread 1)))        ;read drag mode coordinates
 (dosm scp sc col)                   ;put the menu at current point
 (defun *ERROR* (s)                  ;function error
  (princ "\nFunction cancelled!")    ;error message
  (dosm scp sc 0)                    ;erase the menu if error occurred.
   (setq *ERROR* nil)
 )                                   ;end function error
 (setq ppp (grread))
 (setq px (car (cadr ppp)))
 (setq py (cadr (cadr ppp)))         ;get coordinates of selected point
  (dosm scp sc 0)                     ;erase the menu
 (if (and (and (> px x5)
           (> py ly)
          )
      (and (< px x4)
       (< py hy)
     )
    )                                ;if inside the menu box
 (if (< py y1)                       ;if lower row 
  (if (< px x1)                       
   (progn                            
    (princ "perpendicular -")       
    (setvar "OSMODE" 128)
   )                                ;if first column - PER
   (if (< px x2)                   
    (progn
    (princ "node -")
    (setvar "OSMODE" 8)
   )                                ;if second column - NOD
   (if (< px x3)                    
    (progn
      (princ "center -")
      (setvar "OSMODE" 4)
     )                              ;if third column - CEN
     (progn
      (princ "insertion point -")
      (setvar "OSMODE" 64)
     )                              ;if fourth column - INS
    )
   ) 
  )
  (if (< px lx)
   (progn
    (princ "tangent -")
    (setvar "OSMODE" 256)
   )                                ;if upper row and first column - END
   (if (< px x1)
    (progn
     (princ "endpoint -")
     (setvar "OSMODE" 1)
    )                              ;if second column - INT
    (if (< px x2)
     (progn
      (princ "intersection -")
      (setvar "OSMODE" 32)
     )                             ;if third column - NEA
     (if (< px x3)
      (progn
      (princ "nearest -")
      (setvar "OSMODE" 512)
     )
     (if (< px hx)
      (progn
      (princ "midpoint -")
      (setvar "OSMODE" 2)
     ) 
     (if (< px x4)
     (progn
      (princ "quadrant -")
      (setvar "OSMODE" 16)
        )
       )
      )
     )                             ;if fourth column - MID
    )
   )
  )
 )
 (progn
  (princ "Osnap mode set to NONE. -")
  (setvar "OSMODE" 0)
  )                                  ;if outside the menu - NONE
 )
)

(defun MD (px1 py1 px2 py2)        ;if function draw mid menu
 (grdraw (list px1 py1) (list px2 py2) col)
 (cross px1 py1 (* d4 2) (* d4 2))         
 )

(defun TN (px1 py1 px2 py2)        ;if function draw mid menu
  (grdraw (list (+ px1 (* d4 2)) (+ py1 d4)) (list (+ px1 (* d4 2))
  (+ (+ py1 d4) (* d4 2))) col) 
  (grdraw (list (+ px1 d4) (+ (+ py1 d4) (* d4 2))) 
  (list (+ (+ px1 d4) (* d4 2)) (+ (+ py1 d4) (* d4 2))) col)

 )

(defun QD (px1 py1 px2 py2)        ;if function draw mid menu
 (grdraw (list (+ px1 d4) (+ py1 d4)) (list (+ (+ px1 d4) (* d4 2))
  (+ py1 d4)) col) 
 (grdraw (list (+ px1 (* d4 3)) (+ py1 d4)) (list (+ px1 (* d4 3))
  (+ (+ py1 d4) (* d4 2))) col) 
  (grdraw (list (+ px1 (* d4 1)) (+ py1 d4)) (list (+ px1 (* d4 1))
  (+ (+ py1 d4) (* d4 2))) col) 
(grdraw (list (+ px1 d4) (+ (+ py1 d4) (* d4 2)))
  (list (+ (+ px1 d4) (* d4 2)) (+ (+ py1 d4) (* d4 2))) col)
  (cross px1 py1 (* d4 3) (* d4 1))

)

(defun EN (px1 py1 px2 py2)        ;function draw end menu
 (grdraw (list px1 py1) (list (- px2 d4) (- py2 d4)) col) 
 (cross px1 py1 (* d4 3) (* d4 3))                        
)

(defun IT (px1 py1 px2 py2)       ;function draw int menu
 (grdraw (list px1 py1) (list px2 py2) col) 
 (grdraw (list px1 py2) (list px2 py1) col) 
 (cross px1 py1 (* d4 2) (* d4 2))
)

(defun PP (px1 py1 px2 py2)       ;function draw per menu
 (grdraw (list px1 py1) (list px2 py2) col) 
 (grdraw (list px1 py2) (list (+ px1 (* d4 2)) 
  (+ py1 (* d4 2))) col)
)

(defun NE (px1 py1 px2 py2)      ;function draw near menu
 (grdraw (list px1 py1) (list px2 py2) col) 
 (cross px1 py1 (* d4 2) (* d4 3))
)

(defun CN (px1 py1 px2 py2)         ;function draw cen menu
 (grdraw (list px1 (+ py1 d4)) (list (+ px1 d4) py1) col) 
 (grdraw (list (- px2 d4) py1) (list px2 (+ py1 d4)) col) 
 (grdraw (list px2 (- py2 d4)) (list (- px2 d4) py2) col)  	
 (grdraw (list px1 (- py2 d4)) (list (+ px1 d4) py2) col) 
)

(defun CROSS (px1 py1 dx dy)    ;function draw cross (Draws Horiz.Lines
 (grdraw (list (- (+ px1 dx) (/ d4 2.0)) (+ py1 dy)) 
   (list (+ (+ px1 dx) (/ d4 2.0)) (+ py1 dy)) col)
 (grdraw (list (+ px1 dx) (- (+ py1 dy) (/ d4 2.0))) 
   (list (+ px1 dx) (+ (+ py1 dy) (/ d4 2.0))) col)  
)

(defun ND (px1 py1 px2 py2 / d8)    ;function draw node menu
 (setq d8 (/ d4 2.0))
 (grdraw (list (- (+ px1 (* d4 2)) (/ d8 4.0)) (+ py1 (* d4 2)))
   (list (+ (+ px1 (* d4 2)) (/ d8 2)) (+ py1 (* d4 2))) col)
 (grdraw (list (+ px1 (* d4 2)) (- (+ py1 (* d4 2)) (/ d8 4.0)))
   (list (+ px1 (* d4 2)) (+ (+ py1 (* d4 2)) (/ d8 2))) col) 
)

(defun IN (px1 py1 px2 py2)    ;function draw ins menu
 (grdraw (list (+ px1 d4) (+ py1 d4)) (list (+ (+ px1 d4) (* d4 2))
  (+ py1 d4)) col) 
 (grdraw (list (+ px1 (* d4 2)) (+ py1 d4)) (list (+ px1 (* d4 2))
  (+ (+ py1 d4) (* d4 2))) col) 
(grdraw (list (+ px1 d4) (+ (+ py1 d4) (* d4 2))) 
 (list (+ (+ px1 d4) (* d4 2)) (+ (+ py1 d4) (* d4 2))) col)
)

(princ "OSM10 Loaded!")
(prin1)




 